@@ -1,4 +1,4 @@
-use num::complex::Complex;
+use numbers::*;
 use super::Zero;
 use std::mem;
 
@@ -65,6 +65,19 @@ pub trait SimdGeneric<T>: Simd<T>
     fn store(self, array: &mut [T], index: usize);
 }
 
+pub trait SimdApproximations<T> : Simd<T>
+    where T: Sized + Sync + Send {
+    fn ln_approx(self) -> Self;
+
+    fn exp_approx(self) -> Self;
+
+    fn sin_approx(self) -> Self;
+
+    fn cos_approx(self) -> Self;
+
+    fn sin_cos_approx(self, is_sin: bool) -> Self;
+}
+
 #[repr(packed)]
 #[derive(Debug, Copy, Clone)]
 struct Unalign<T>(T);
@@ -78,7 +91,7 @@ macro_rules! simd_generic_impl {
                 $reg::splat(0.0)
             }
         }
-    
+
         impl SimdGeneric<$data_type> for $reg {
             #[inline]
             fn calc_data_alignment_reqs(array: &[$data_type]) -> (usize, usize, usize) {
@@ -193,7 +206,7 @@ macro_rules! simd_generic_impl {
 mod avx;
 
 #[cfg(any(feature = "doc", feature="use_avx"))]
-pub use self::avx::{Reg32, Reg64};
+pub use self::avx::{Reg32, Reg64, IntReg32, IntReg64, UIntReg32, UIntReg64};
 
 #[cfg(any(feature = "doc", all(feature = "use_sse", not(feature = "use_avx"))))]
 mod sse;
@@ -201,17 +214,18 @@ mod sse;
 #[cfg(any(feature = "doc", all(feature = "use_sse", not(feature = "use_avx"))))]
 pub use self::sse::{Reg32, Reg64, IntReg32, IntReg64, UIntReg32, UIntReg64};
 
+//#[cfg(any(feature = "doc", any(feature = "use_sse", feature = "use_avx")))]
 #[cfg(any(feature = "doc", all(feature = "use_sse", not(feature = "use_avx"))))]
 mod approximations;
 
-#[cfg(any(feature = "doc", all(feature = "use_sse", not(feature = "use_avx"))))]
-pub use self::approximations::*;
-
 #[cfg(any(feature = "doc", not(any(feature = "use_avx", feature="use_sse"))))]
 mod fallback;
 
 #[cfg(any(feature = "doc", not(any(feature = "use_avx", feature="use_sse"))))]
 pub use self::fallback::{Reg32, Reg64};
 
+#[cfg(any(feature = "doc", not(feature="use_sse")))]
+mod approx_fallback;
+
 simd_generic_impl!(f32, Reg32);
 simd_generic_impl!(f64, Reg64);
