@@ -1,9 +1,9 @@
 use std::mem;
 use std::ptr;
-use RealNumber;
-use num::Complex;
+use {RealNumber, array_to_complex_mut, memcpy, memzero};
+use numbers::*;
 use multicore_support::*;
-use super::super::{array_to_complex_mut, VoidResult, Buffer, Owner, ErrorReason, NumberSpace,
+use super::super::{VoidResult, Buffer, Owner, ErrorReason, NumberSpace,
                    Domain, ResizeOps, DspVec, Vector, ToSliceMut, MetaData};
 
 /// This trait allows to reorganize the data by changing positions of the individual elements.
@@ -343,10 +343,7 @@ impl<S, T, N, D> InsertZerosOps<T> for DspVec<S, T, N, D>
         match option {
             PaddingOption::End => {
                 // Zero target
-                let dest = &mut data[len_before] as *mut T;
-                unsafe {
-                    ptr::write_bytes(dest, 0, len - len_before);
-                }
+                memzero(data, len_before .. len);
                 Ok(())
             }
             PaddingOption::Surround => {
@@ -358,17 +355,11 @@ impl<S, T, N, D> InsertZerosOps<T> for DspVec<S, T, N, D>
                     left *= 2;
                 }
 
-                unsafe {
-                    let src = &data[0] as *const T;
-                    let dest = &mut data[left] as *mut T;
-                    ptr::copy(src, dest, len_before);
-                    if right > 0 {
-                        let dest = &mut data[len - right] as *mut T;
-                        ptr::write_bytes(dest, 0, right);
-                    }
-                    let dest = &mut data[0] as *mut T;
-                    ptr::write_bytes(dest, 0, left);
+                memcpy(data, 0..len_before, left);
+                if right > 0 {
+                    memzero(data, len - right .. len);
                 }
+                memzero(data, 0 .. left);
                 Ok(())
             }
             PaddingOption::Center => {
@@ -381,19 +372,10 @@ impl<S, T, N, D> InsertZerosOps<T> for DspVec<S, T, N, D>
                 }
 
                 if right > 0 {
-                    unsafe {
-                        let src = &data[left] as *const T;
-                        let dest = &mut data[len - right] as *mut T;
-                        // src and dest may overload with copy, a unit test covers this case
-                        ptr::copy(src, dest, right);
-                    }
+                    memcpy(data, left..left + right, len - right);
                 }
 
-                unsafe {
-                    let dest = &mut data[left] as *mut T;
-                    ptr::write_bytes(dest, 0, len - left - right);
-
-                }
+                memzero(data, left .. len - right);
                 Ok(())
             }
         }
@@ -462,55 +444,37 @@ impl<S, T, N, D> InsertZerosOpsBuffered<S, T> for DspVec<S, T, N, D>
             match option {
                 PaddingOption::End => {
                     // Zero target
-                    let src = &data[0] as *const T;
-                    let dest_start = &mut target[0] as *mut T;
-                    let dest_end = &mut target[len_before] as *mut T;
-                    unsafe {
-                        ptr::copy(src, dest_start, len_before);
-                        ptr::write_bytes(dest_end, 0, len - len_before);
-                    }
+                    &mut target[0..len_before].copy_from_slice(&data[0..len_before]);
+                    memzero(target, len_before .. len);
                 }
                 PaddingOption::Surround => {
                     let diff = (len - len_before) / if is_complex { 2 } else { 1 };
-                    let mut right = (diff - 1) / 2;
+                    let mut right = (diff) / 2;
                     let mut left = diff - right;
                     if is_complex {
                         right *= 2;
                         left *= 2;
                     }
 
-                    unsafe {
-                        let src = &data[0] as *const T;
-                        let dest = &mut target[left] as *mut T;
-                        ptr::copy(src, dest, len_before);
-                        if right > 0 {
-                            let dest = &mut target[len - right] as *mut T;
-                            ptr::write_bytes(dest, 0, right);
-                        }
-                        let dest = &mut target[0] as *mut T;
-                        ptr::write_bytes(dest, 0, left);
+                    &mut target[left..left+len_before].copy_from_slice(&data[0..len_before]);
+                    if right > 0 {
+                        memzero(target, len - right .. len);
                     }
+                    memzero(target, 0 .. left);
                 }
                 PaddingOption::Center => {
-                    let mut diff = (len - len_before) / if is_complex { 2 } else { 1 };
-                    let mut right = (diff - 1) / 2;
-                    let mut left = diff - right ;
+                    let step = if is_complex { 2 } else { 1 };
+                    let points_before = len_before / step;
+                    let mut right = points_before / 2;
+                    let mut left = points_before - right;
                     if is_complex {
                         right *= 2;
                         left *= 2;
-                        diff *= 2;
                     }
 
-                    unsafe {
-                        let src = &data[len_before / 2 + 1] as *const T;
-                        let dest = &mut target[len - right] as *mut T;
-                        ptr::copy(src, dest, right);
-                        let src = &data[0] as *const T;
-                        let dest = &mut target[0] as *mut T;
-                        ptr::copy(src, dest, left);
-                        let dest = &mut target[left] as *mut T;
-                        ptr::write_bytes(dest, 0, len - diff);
-                    }
+                    &mut target[len - right .. len].copy_from_slice(&data[len_before - right .. len_before]);
+                    &mut target[0 .. left].copy_from_slice(&data[0 .. left]);
+                    memzero(target, left .. len - len_before);
                 }
             }
         }
@@ -719,10 +683,8 @@ mod tests {
         let mut v = vec![1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 11.0, 12.0].to_complex_time_vec();
         let mut buffer = SingleBuffer::new();
         v.zero_pad_b(&mut buffer, 10, PaddingOption::Surround);
-        // The expected result is required so that the convolution theorem holds true
-        // (mul in freq is the same as conv)
-        let expected = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0,
-                        10.0, 11.0, 12.0, 0.0, 0.0];
+        let expected = [0.0, 0.0, 0.0, 0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0,
+                        10.0, 11.0, 12.0, 0.0, 0.0, 0.0, 0.0];
         assert_eq!(&v[..], &expected);
     }
 
