@@ -1,5 +1,5 @@
-use RealNumber;
-use super::super::{array_to_complex_mut, Vector, DspVec, ToSliceMut, Domain, NumberSpace, MetaData};
+use {RealNumber, array_to_complex_mut};
+use super::super::{Vector, DspVec, ToSliceMut, Domain, NumberSpace, MetaData};
 
 /// A trait to calculate the diff (1st derivative in a discrete number space) or cumulative sum 
 /// (integral  in a discrete number space).
