@@ -1,8 +1,8 @@
-use RealNumber;
-use num::Complex;
+use {RealNumber, array_to_complex};
+use numbers::*;
 use multicore_support::*;
 use simd_extensions::*;
-use super::super::{array_to_complex, Vector, DspVec, ToSlice, Domain, RealNumberSpace,
+use super::super::{Vector, DspVec, ToSlice, Domain, RealNumberSpace,
                    ComplexNumberSpace};
 
 #[repr(C)]
@@ -39,9 +39,9 @@ pub trait StatisticsOps<T>: Sized
     /// # Example
     ///
     /// ```
-    /// # extern crate num;
+    /// # extern crate num_complex;
     /// # extern crate basic_dsp_vector;
-    /// # use num::complex::Complex32;
+    /// # use num_complex::Complex32;
     /// use basic_dsp_vector::*;
     /// # fn main() {
     /// let vector = vec!(1.0, 2.0, 3.0, 4.0, 5.0, 6.0).to_complex_time_vec();
@@ -67,9 +67,9 @@ pub trait StatisticsOps<T>: Sized
     /// # Example
     ///
     /// ```
-    /// # extern crate num;
+    /// # extern crate num_complex;
     /// # extern crate basic_dsp_vector;
-    /// # use num::complex::Complex32;
+    /// # use num_complex::Complex32;
     /// use basic_dsp_vector::*;
     /// # fn main() {
     /// let vector = vec!(1.0, 2.0, 3.0, 4.0, 5.0, 6.0).to_complex_time_vec();
@@ -92,9 +92,9 @@ pub trait SumOps<T>: Sized
     /// # Example
     ///
     /// ```
-    /// # extern crate num;
+    /// # extern crate num_complex;
     /// # extern crate basic_dsp_vector;
-    /// # use num::complex::Complex32;
+    /// # use num_complex::Complex32;
     /// use basic_dsp_vector::*;
     /// # fn main() {
     /// let vector = vec!(1.0, 2.0, 3.0, 4.0, 5.0, 6.0).to_complex_time_vec();
@@ -108,9 +108,9 @@ pub trait SumOps<T>: Sized
     /// # Example
     ///
     /// ```
-    /// # extern crate num;
+    /// # extern crate num_complex;
     /// # extern crate basic_dsp_vector;
-    /// # use num::complex::Complex64;
+    /// # use num_complex::Complex64;
     /// use basic_dsp_vector::*;
     /// # fn main() {
     /// let vector = vec!(1.0, 2.0, 3.0, 4.0, 5.0, 6.0).to_complex_time_vec();
@@ -366,7 +366,7 @@ impl<S, T, N, D> StatisticsOps<Statistics<T>> for DspVec<S, T, N, D>
             stats
         });
 
-        Statistics::merge(&chunks)
+        Statistics::merge(&chunks[..])
     }
 
     fn statistics_split(&self, len: usize) -> Vec<Statistics<T>> {
@@ -393,7 +393,7 @@ impl<S, T, N, D> StatisticsOps<Statistics<T>> for DspVec<S, T, N, D>
             results
         });
 
-        Statistics::merge_cols(&chunks)
+        Statistics::merge_cols(&chunks[..])
     }
 
     fn statistics_splitted(&self, len: usize) -> Vec<Statistics<T>> {
@@ -426,7 +426,7 @@ impl<S, T, N, D> SumOps<T> for DspVec<S, T, N, D>
                 }
                 sum
             });
-            chunks.iter()
+            (&chunks[..]).iter()
                 .map(|v| v.sum_real())
                 .fold(T::zero(), |a, b| a + b)
         } else {
@@ -460,7 +460,7 @@ impl<S, T, N, D> SumOps<T> for DspVec<S, T, N, D>
                 }
                 sum
             });
-            chunks.iter()
+            (&chunks[..]).iter()
                 .map(|v| v.sum_real())
                 .fold(T::zero(), |a, b| a + b)
         } else {
@@ -501,7 +501,7 @@ impl<S, T, N, D> StatisticsOps<Statistics<Complex<T>>> for DspVec<S, T, N, D>
             stat
         });
 
-        Statistics::merge(&chunks)
+        Statistics::merge(&chunks[..])
     }
 
     fn statistics_split(&self, len: usize) -> Vec<Statistics<Complex<T>>> {
@@ -529,7 +529,7 @@ impl<S, T, N, D> StatisticsOps<Statistics<Complex<T>>> for DspVec<S, T, N, D>
             results
         });
 
-        Statistics::merge_cols(&chunks)
+        Statistics::merge_cols(&chunks[..])
     }
 
     fn statistics_splitted(&self, len: usize) -> Vec<Statistics<Complex<T>>> {
@@ -562,7 +562,7 @@ impl<S, T, N, D> SumOps<Complex<T>> for DspVec<S, T, N, D>
                 }
                 sum
             });
-            chunks.iter()
+            (&chunks[..]).iter()
                 .map(|v| v.sum_complex())
                 .fold(Complex::<T>::new(T::zero(), T::zero()), |acc, x| acc + x)
         } else {
@@ -596,7 +596,7 @@ impl<S, T, N, D> SumOps<Complex<T>> for DspVec<S, T, N, D>
                 }
                 sum
             });
-            chunks.iter()
+            (&chunks[..]).iter()
                 .map(|v| v.sum_complex())
                 .fold(Complex::<T>::new(T::zero(), T::zero()), |acc, x| acc + x)
         } else {
