@@ -1,10 +1,10 @@
 //! This module defines the basic vector trait and indexers.
-use RealNumber;
-use super::{array_to_complex, array_to_complex_mut, DspVec, NumberSpace, ComplexNumberSpace,
+use {RealNumber, array_to_complex, array_to_complex_mut};
+use super::{DspVec, NumberSpace, ComplexNumberSpace,
             Domain, DataDomain, ToSlice, ToSliceMut, ErrorReason, VoidResult, TypeMetaData};
 use multicore_support::MultiCoreSettings;
 use std::ops::*;
-use num::complex::Complex;
+use numbers::*;
 
 /// Like [`std::ops::Index`](https://doc.rust-lang.org/std/ops/trait.Index.html)
 /// but with a different method name so that it can be used to implement an additional range
